# -*- coding: utf-8 -*-
'''
    A module for startup settings
'''

from __future__ import absolute_import

import logging
import os.path
import sys

# Import local libs
# This file may be loaded out of __pycache__, so the
# directory of its .py may not be in the search path.
IMPORT_PATH = os.path.dirname(__file__)
if IMPORT_PATH.endswith('__pycache__'):
    IMPORT_PATH = os.path.dirname(IMPORT_PATH)
sys.path.append(IMPORT_PATH)
try:
    import _nisysmgmt_grains  # pylint: disable=import-error,3rd-party-module-not-gated
finally:
    # Remove the extra search path that we added to sys.path
    del sys.path[-1:]

log = logging.getLogger(__name__)

try:
    import salt.modules.cmdmod as cmd
    import salt.serializers.json as JSON
except ImportError:
    log.critical("Salt is not available")

# Define the module's virtual name
__virtualname__ = 'startup'

NIRTCFG_PATH = '/usr/local/natinst/bin/nirtcfg'
FWSETENV_PATH = '/sbin/fw_setenv'


def __virtual__():
    '''
        Only load this module if the nirtcfg command exist and is older NILinuxRT
        :return: True if environment is set up and False otherwise
    '''
    if __grains__['os_family'] == 'NILinuxRT' and __salt__['nilrt.is_older_nilrt']() and os.path.exists(NIRTCFG_PATH):
        return True
    return False, 'The startup_settings module cannot be loaded.'


def get_all(json_format=False):
    '''
        .. note::
            Get all of these systemsettings:
                - NoFPGAApp
                - NoApp
                - ConsoleOut
        :param json_format: If true, returns the result in JSON format
        :return: Returns settings
        CLI Example:

        .. code-block:: bash
        salt '*' startup.get_all
        salt '*' startup.get_all True
    '''
    settings = {'NoFPGAApp': 'NoFPGAApp.enabled',
                'NoApp': 'NoApp.enabled',
                'ConsoleOut': 'ConsoleOut.enabled'}
    for key, value in settings.items():
        settings[key] = cmd.run('{0} --get section=systemsettings,token={1}'.format(NIRTCFG_PATH, value))
    if not json_format:
        return settings
    return JSON.serialize(settings)


def get(setting):
    '''
        .. note::
            Get one of these systemsettings:
                - NoFPGAApp
                - NoApp
                - ConsoleOut
        :param setting: Name of setting.
        :return: Returns value of that setting or -1 if error.
        CLI Example:

        .. code-block:: bash
        salt '*' startup.setting noapp
    '''
    setting = setting.strip().lower()
    settings = {'nofpgaapp': 'NoFPGAApp.enabled',
                'noapp': 'NoApp.enabled',
                'consoleout': 'ConsoleOut.enabled'}
    try:
        return cmd.run('{0} --get section=systemsettings,token={1}'.format(NIRTCFG_PATH, settings[setting]))
    except KeyError:
        return -1


def enable_console_out(enable=True):
    '''
        .. note::
            Enable or disable ConsoleOut
        :param enable: If true enable ConsoleOut else disable ConsoleOut. Default is True.
        CLI Example:

        .. code-block:: bash
        salt '*' startup.enable_console_out
    '''
    cmd.run('{0} --set section=systemsettings,token={1},value={2}'.format(NIRTCFG_PATH, 'ConsoleOut.enabled', enable))
    cmd.run('{0} consoleoutenable={1}'.format(FWSETENV_PATH, enable))
    system_settings = _nisysmgmt_grains.get_last_known_startup_settings()
    system_settings['ConsoleOut'] = str(enable)
    __salt__['event.fire']({'force_refresh': True}, 'grains_refresh')
    return True


def enable_no_fpga_app(enable=True):
    '''
        .. note::
            Enable or disable NoFPGAApp
        :param enable: If true enable NoFPGAApp else disable NoFPGAApp. Default is True.
        CLI Example:

        .. code-block:: bash
        salt '*' startup.enable_no_fpga_app
    '''
    cmd.run('{0} --set section=systemsettings,token={1},value={2}'.format(NIRTCFG_PATH, 'NoFPGAApp.enabled', enable))
    system_settings = _nisysmgmt_grains.get_last_known_startup_settings()
    system_settings['NoFPGAApp'] = str(enable)
    __salt__['event.fire']({'force_refresh': True}, 'grains_refresh')
    return True


def enable_no_app(enable=True):
    '''
        .. note::
            Enable or disable NoApp
        :param enable: If true enable NoApp else disable NoApp. Default is True.
        CLI Example:

        .. code-block:: bash
        salt '*' startup.enable_no_app
    '''
    cmd.run('{0} --set section=systemsettings,token={1},value={2}'.format(NIRTCFG_PATH, 'NoApp.enabled', enable))
    system_settings = _nisysmgmt_grains.get_last_known_startup_settings()
    system_settings['NoApp'] = str(enable)
    __salt__['event.fire']({'force_refresh': True}, 'grains_refresh')
    return True
